<?php
/**
 * $Horde: forwards/lib/Driver/qmail.php,v 1.3.2.1 2003/01/20 05:17:57 ericr Exp $
 *
 * Copyright 2001-2003 Eric Rostetter <eric.rostetter@physics.utexas.edu>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 *
 * Forwards_Driver_qmail:: implements the Forwards_Driver API for ftp driven
 * qmail mail servers.  
 *
 * @author  Eric Rostetter <eric.rostetter@physics.utexas.edu>
 * @version $Revision: 1.3.2.1 $ 
 * @since   Forwards 2.2
 * @package forwards
 */

require_once HORDE_BASE . '/lib/VFS.php';

class Forwards_Driver_qmail extends Forwards_Driver {

    /** The FTP stream we open via the Horde_VFS class */
    var $_vfs;

    /** The error string returned to the user if an error occurs. */
    var $err_str;

    /** Hash containing configuration data. */
    var $params;

/**
 * Constructs a new ftp qmail Forwards_Driver object.
 *
 * @param array  $params    A hash containing connection parameters.
 */
function Forwards_Driver_qmail($params = array())
{
    $this->params = $params;
}

/**
 * Check if the realm has a specific configuration.  If not, try to fall
 * back on the default configuration.  If still not a valid configuration
 * then exit with an error.
 *
 * @param string    $realm      The realm of the user, or "default" if none.
 *                              Note: passed by reference so we can change
 *                              it's value!
 *
 */

function check_config(&$realm) {

    // If no realm passed in, or no host config for the realm passed in,
    // then we fall back to the default realm

    if ( empty($realm) || empty($this->params[$realm]['host']) ) {
       $realm = "default";
    }

    // If still no host/port, then we have a misconfigured module
    if (empty($this->params[$realm]['host']) ||
        empty($this->params[$realm]['port']) ||
        empty($this->params[$realm]['user']) ||
        empty($this->params[$realm]['pass']) ||
        empty($this->params[$realm]['root']) ) {
            $this->err_str = _("The module is not properly configured!");
            return false;
    }
    return true;
}

/**
 * Begins forwarding of mail for a user.
 *
 * @param string    $user       The username to enable forwarding for.
 * @param string    $realm      The realm of the user, or "default" if none.
 * @param string    $target     The email address that mail should be
 *                              forwarded to.
 * @param optional boolean $keeplocal A flag that, when true, causes a copy of
 *                                    forwarded email to be kept in the local
 *                                    mailbox.
 *
 * @return boolean  Returns true on success, false on error.
 */

    function enableForwarding($user, $realm = 'default', $pass, $where,
                              $metoo = false) {

        // Make sure the configuration file is correct
        if (!$this->check_config($realm)) {
            return false;
        }

        // Build the ftp array to pass to VFS
        $_args = array( 'hostspec' => $this->params[$realm]['host'],
                        'port' => $this->params[$realm]['port'],
                        'username' => $this->params[$realm]['user'],
                        'password' => $this->params[$realm]['pass'] );

        // Create the VFS ftp driver
        $_vfs = &Horde_VFS::singleton('ftp', $_args);
        $_vfs->setParams($_args);

        // Try to login with the username/password, no realm
        // This isn't really needed, but allows for a better error message
        $status = $_vfs->checkCredentials();
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            $this->err_str .= '  ' .  _("Check your username and password");
            return false;
        }

        // Create the forwarding information
        $address = '&' . $where;
        if ($metoo == "on") {
           $address .= "\n./Maildir/";
        }

	$splitted = split("@", Auth::getAuth());
	$name = @$splitted[0]; $domain = @$splitted[1];
	$path = $this->params[$realm]['root'] . '/' . $domain . '/' . $name;

        // Set the forward
        $status = $_vfs->writeData($path, '.qmail', $address);
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }

        // Try to change the permissions, but ignore any errors
        $_vfs->changePermissions($path, '.qmail', '0600');

        $_vfs->_disconnect();
        return true;
    }

/**
 * Stops forwarding of mail for a user.
 *
 * @param string    $user       The username of the user to disable forwarding.
 * @param string    $realm      The realm of the user.
 * @param string    $pass       The password of the user.
 *
 * @return boolean  Returns true on success, false on error.
 */

    function disableForwarding($user, $realm = 'default', $pass) {
        if (!$this->check_config($realm)) {
           return false;
        }

        $_args = array( 'hostspec' => $this->params[$realm]['host'],
                        'port' => $this->params[$realm]['port'],
                        'username' => $this->params[$realm]['user'],
                        'password' => $this->params[$realm]['pass'] );

        $_vfs = &Horde_VFS::singleton('ftp', $_args);
        $_vfs->setParams($_args);

        // Try to login with the username/password, no realm
        $status = $_vfs->checkCredentials();
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            $this->err_str .= '  ' .  _("Check your username and password");
            return false;
        }

	$splitted = split("@", Auth::getAuth());
	$name = @$splitted[0]; $domain = @$splitted[1];
	$path = $this->params[$realm]['root'] . '/' . $domain . '/' . $name;

        $status = $_vfs->deleteFile($path, '.qmail');
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            $this->err_str .= '  ' .  _("Maybe you didn't have a forward enabled?");
            return false;
        }
        $_vfs->_disconnect();
        return true;
    }

/**
 * Retrieves current content of mail redirection file
 *
 * @param string    $user       The username of the user to get forward of.
 * @param string    $realm      The realm of the user.
 *
 * @return mixed    A string with the content of forwarding mail file or false.
 */

    function getContent($user, $realm = 'default', $pass) {

        // Make sure the configuration file is correct
        if (!$this->check_config($realm)) {
            return false;
        }

        // Build the ftp array to pass to VFS
        $_args = array( 'hostspec' => $this->params[$realm]['host'],
                        'port' => $this->params[$realm]['port'],
                        'username' => $this->params[$realm]['user'],
                        'password' => $this->params[$realm]['pass'] );

        // Create the VFS ftp driver
        $_vfs = &Horde_VFS::singleton('ftp', $_args);
        $_vfs->setParams($_args);

        // Try to login with the username/password, no realm
        $status = $_vfs->checkCredentials();
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            $this->err_str .= '  ' .  _("Check your username and password");
            return false;
        }

	$splitted = split("@", Auth::getAuth());
	$name = @$splitted[0]; $domain = @$splitted[1];
	$path = $this->params[$realm]['root'] . '/' . $domain . '/' . $name;

        $status = $_vfs->read($path, '.qmail');
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }
	
        return $status;
    }

/**
 * Retrieves current target of mail redirection
 *
 * @param string    $user       The username of the user to get forward of.
 * @param string    $realm      The realm of the user.
 *
 * @return mixed    A string of current forwarding mail address, or false.
 */

    function currentTarget($user, $realm = 'default', $pass) {

	$status = $this->getContent($user, $realm, $pass);

	if ($status)
	{
	   if ($status[0] != '&')
	      return false;

	   // Remove the ampersand (&)
	   $splitted = split("&", $status);
	   $status = @$splitted[1];
	   
	   // Remove the possible local delivery
	   $splitted = split("\n", $status);
	   $status = @$splitted[0];
	}

        return $status;
    }

/**
 * Retrieves current state of mail redirection
 *
 * @param string    $user       The username to check forwarding for.
 * @param string    $realm      The realm of the user.
 * @return boolean  Returns true/false based on if forwarding is enabled.
 */

/* This function is implemented poorly, and should be rewritten */

    function isEnabledForwarding($user, $realm, $password) {
        return $this->currentTarget($user, $realm, $password) != false;
    }

/**
 * Retrieves true if a local copy of forwarded messages is being kept
 *
 * @param string    $user       The username of the user to check.
 *
 * @param string    $realm      The realm of the user to check.
 *
 * @return boolean    Returns true if retian copy is enabled for the user else false.
 *
 */
    function isKeepLocal($user, $realm = 'default', $password) {
       
       $status = $this->getContent($user, $realm, $password);
       
       if ($status)
       {
	  // Get the possible second line
	  $splitted = split("\n", $status);
	  $metoo = @$splitted[1];

	  if (@strcmp("./Maildir/", $metoo) == 0)
	     $status = true;
	  else
	     $status = false;
       }
       
       return $status;
    }
    
}

?>
